<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

require_once 'dataTableSql.php';
require_once 'engine/dataTableSql_oci8.php';
require_once 'engine/dataTableSql_mysql.php';
require_once 'engine/dataTableSql_engine.php';
require_once 'dataTableStructure.php';

require_once 'lib/customlib/database/DbConfig.class.php';
require_once 'lib/customlib/common/date_conversion/DateConversion.php';

/**
 * Description of dataTableDBclass para ORACLE, con conector oci8
 *
 * @author gustavo
 */
class dataTableDB {

    const LENGTH_VAR_GET = "length";
    const START_VAR_GET = "start";
    const DRAW_VAR_GET = "draw";
    const ORDER_VAR_GET = "order";
    const ORDER_COLUMN_VAR_GET = "column";
    const ORDER_DIR_VAR_GET = "dir";
    const COLUMNS_VAR_GET = "columns";

    /** @var dataTableStructure  */
    private $dtStructure;
    /** @var dataTableSql */
    private $dtSql;
    private $entity;

    // Output properties
    private $resultSet;
    private $sOutput;
    private $iTotal;
    private $iFilteredTotal;

    // Much rows properties
    private $muchRows = false;
    private $maxRows_toProcess = 9999999;
    private $disableCounter = false;

    private $draw;
    private $columnList;

    public function dataTableDB() {
        if (DbConfig::isOCI8())
            $this->dtSql = new dataTableSql_oci();
        else
            $this->dtSql = new dataTableSql_mysql();
    }

    public function setEntity($entity){
        $this->entity = $entity;
        $entityObj = new $entity();
        $this->dtStructure = new dataTableStructure($entityObj->tableName);
    }

    /**
     * Setea un query que se utiliza como tabla base para la consulta
     * del datatable
     *
     * @param string $query
     */
    public function setBaseTableQuery($query){
        $this->dtStructure->setBaseTableQuery($query);
    }

    /**
     * Deshabilita el contador de filas obtenidas.
     * Setea un numero grande en las filas totales y filtradas.
     */
    public function setMuchRows() {
        $this->muchRows = true;
        $this->disableCounter = true;

        $this->iTotal = $this->maxRows_toProcess;
        $this->iFilteredTotal = $this->maxRows_toProcess;
    }

    /**
     * Instancia el orden, filtros y cant. de filas a partir
     * del get.
     */
    private function setVarsFromGet() {

        $this->draw = intval(Secure_PostGet_Vars::get_Get(self::DRAW_VAR_GET));

        $length = Secure_PostGet_Vars::get_Get(self::LENGTH_VAR_GET);
        if($length)
            $this->dtStructure->setLength($length);

        $start = Secure_PostGet_Vars::get_Get(self::START_VAR_GET);
        if($start)
            $this->dtStructure->setStart($start);

        $orderList = $_GET[self::ORDER_VAR_GET];
        if ( sizeof($orderList) > 0) {

            foreach ($orderList as $order) {

                if (is_int($order[self::ORDER_COLUMN_VAR_GET]))
                    $sortField = intval($order[self::ORDER_COLUMN_VAR_GET]);
                else
                    $sortField = $order[self::ORDER_COLUMN_VAR_GET];

                $sortOrder = $order[self::ORDER_DIR_VAR_GET];
                $this->dtStructure->addSortingCol($sortField,$sortOrder);
            }
        }

        $_column_get_params = $_GET[self::COLUMNS_VAR_GET];

        $this->columnList = new dtColumnList($_column_get_params);

        if ($this->columnList) {

            foreach ($this->columnList->getColumnList() as $ndx=>$field) {

                // $filterValue = $this->columnList->getColumnFilterValue($i);
                $filterValue = $field->getSearchExpression();
                $fieldName = $field->getData();

                if (!empty($filterValue) or $filterValue==="0")
                    $this->dtStructure->addFilterFieldColumn($fieldName, $filterValue);

            }
        }

    }

    public function processData() {
        $this->setVarsFromGet();

        if ($length = $this->dtStructure->getLength()) {
            $start = $this->dtStructure->getStart();

            $this->dtSql->setLimit($length, $start);
        }

        $_query = $this->dtStructure->getQuery();       

        $this->resultSet = $this->dtSql->ExecuteQuery($_query);

        if($this->resultSet->getStatusResponse()->isOk()){
            if(!$this->disableCounter) {

                $this->iFilteredTotal = $this->dtSql->getCalculateFoundRows();
                $this->iTotal = $this->dtSql->getCalculateTotalRows();
            }
        }

    }

    public function getOutputData() {
        $_ret = statusResponse::getInstanceOk();

        $sOutput = new stdClass();
        $sOutput->draw = $this->draw;
        $sOutput->recordsTotal = $this->iTotal;
        $sOutput->recordsFiltered = $this->iFilteredTotal;

        $aaData = array();
        if($this->resultSet->getStatusResponse()->isOk()) {
            $it = $this->resultSet->getIterator();
            foreach ($it as $aRow) {

                $data = new stdClass();
                foreach ($this->dtStructure->getFieldList() as $field) {
                    $_fieldName = $field->getName();

                    if ($field->getType() != dtTableField::TYPE_DATE && $field->getType() != dtTableField::TYPE_DATE_TIME) {
                        $data->$_fieldName = $aRow[$_fieldName];
                    } else {
                        $value = $field->getFormattedValue($aRow[$_fieldName]);
                        $data->$_fieldName = $value;
                    }
                }

                $aaData[] = $data;
            }

        }else{
            $_ret = $this->resultSet->getStatusResponse();
        }

        $sOutput->data = $aaData;
        $this->sOutput = $sOutput;

        $_ret->setData($sOutput);

        return $_ret->getJsonEncoded();
    }

}

?>
